/* Copyright 2013 Robert Edward Steckroth II <RobertSteckroth@gmail.com> Bust0ut, Surgemcgee

* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; version 3.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import QtQuick.XmlListModel 2.0
import "javascript/graph.js" as Graph

Rectangle {
    property var canvas_object
    onVisibleChanged: visible && displayDayInfo()
    color: "#FCFCFC"
    z: 1
    Item {
        id: chartRect
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        height: parent.height - dayRangeSliderRect.height
        MouseArea {
            anchors.fill: parent
            onPressed: chartLine.visible = true
            onReleased: chartLine.visible = false
            onMouseXChanged: visible && displayDayInfo(mouseX)
        }
        Item {
            id: dayRangeSliderRect
            height: units.gu(3)+5
            anchors.top: parent.bottom
            anchors.left: parent.left
            anchors.right: parent.right
            ListItem.SingleControl {
                anchors.fill: parent
                control:
                    Slider {
                        id: dayRangeSlider
                        anchors.fill: parent
                        minimumValue: 5
                        maximumValue: 730
                        live: false
                        value: chart_day_range
                        onValueChanged: chart_day_range = parseInt(value)
                }
            }
        }
    }
    Item {
        z: 2  // We want this in front of the other stuff
        visible: currentStocksList.count > 0 ? true : false
        anchors.fill: parent
        clip: false
        ActivityIndicator {
             objectName: "chartsActivityIndicator"
             anchors.centerIn: parent
             clip: false
             running: currentStocksChart.status != XmlListModel.Ready
        }
        Image {
            id: chartDot
            objectName: "chartDot"
            z: 2 // These are scoped apparently
            visible: false
            x: Graph.graph.options.width // Using graph.width to avoid state changes (.pragma library is a good thing :)
            source: "svg/chart_dot.svg"
            opacity: 0.89
            sourceSize.height: visible && units.gu(1)+1 // Stop the SVG rasterization if the object is not visible. Use this everyone
            sourceSize.width: visible && height  // Stop the SVG rasterization if the object is not visible. Use this everyone
        }
        Rectangle { //This stuff could have better math and design
              id: chartLine
              z: 1
              color: "#AD9624"
              width: Math.ceil(chartDot.width/5) // Respect the units.gu() stuff
              y: { x; Graph.graph.getChartSetting(0, "chartLineTop") || 0 }           // Be nice if I had a less frequent state changed single than x
              height: { x; Graph.graph.getChartSetting(0, "chartLineBottom")-y || 0 } // but Graph.graph.getChartSetting() is not taxing
              visible: false
              x: chartDot.x+chartDot.width/2-width/2
              opacity: 0.5
        }
        Text {
            id: dayInfoText
            objectName: "dayInfoText"
            x: Graph.graph.options.leftPadding
            text: i18n.tr("")
            color: Graph.graph.options.dateTextColor
            font.pixelSize: FontUtils.sizeToPixels("medium")
        }

    }

    Component.onCompleted: {
        log("Creating Canvas component")
        var component = Qt.createComponent("Stock_Component_Canvas.qml")
        canvas_object = component.createObject(chartsPage)

    }

    function createCanvas() {
            log("createCanvas() called")
            Graph.graph.init(canvas_object.getContext('2d'))
            Graph.newChart(currentStocksChart)
            Graph.graph.options.topPadding = dayInfoText.height // This will keep the dayInfoText from overlapping
            Graph.graph.contextSetSave({lineCap: 'round',
                                        lineWidth: 3,
                                        strokeStyle: 'orange',
                                        font: Math.ceil(FontUtils.sizeToPixels("medium"))+'px sans-serif'
                                       })

    }

    function changeSettings(args) {
        if ( typeof(args) === "string" && typeof(Graph.graph[args]) !== "undefined" ) args = Graph.graph[args]
        for ( var x in args )
            if ( typeof(Graph.graph.options[x]) !== "undefined" ) Graph.graph.options[x] = args[x]
        wipeCanvas()
        repaintCanvas()
    }

    function getSettings(option) {
        if ( typeof(Graph.graph[option]) !== "undefined" ) return Graph.graph[option]
        return Graph.graph.options[option]
    }

    function repaintCanvas() {
        log("repaintCanvas() called")
        chartDot.visible = true
        Graph.graph.options.width = chartRect.width
        Graph.graph.options.height = chartRect.height
        var miliseconds = new Date().getTime()
        Graph.graph.render(0)
        miliseconds = new Date().getTime() - miliseconds
        log("Chart render exec time: "+miliseconds)
        canvas_object.requestPaint()
        displayDayInfo() // no argument will prevent dot placement from moving when resized (uses in same location)
    }

    function wipeCanvas() {
        log("wipeCanvas() called")
        chartDot.visible = false  // Remove this until the canvas is redrawn
        Graph.graph.wipeAll()
    }

    function displayDayInfo(mousex) {
        var new_x_pos = mousex || false
        // info[0] is listModel index
        // info[1] is x position
        // info[2] is y positioncurrentStocksChart
        if ( sub_tab_index !== 1 )
            return
        if ( !new_x_pos )
            new_x_pos = chartDot.x+chartDot.width/2
        var info = Graph.graph.getDayInfo(0, new_x_pos)
        if ( !info ) return
        var chart_index = ( (currentStocksChart.count-1)-info[0] < 0 ) ? 0 : (currentStocksChart.count-1)-info[0] //currentStocksChart.count-1 ) ? currentStocksChart.count-1 : (currentStocksChart.count-1)-info[0]
        if ( chart_index > currentStocksChart.count-1 ) chart_index = currentStocksChart.count-1
        dayInfoText.text = i18n.tr([currentStocksChart.get(chart_index).date.split('-')[1]+"/"+currentStocksChart.get(chart_index).date.split('-')[2]+"/"+currentStocksChart.get(chart_index).date.split('-')[0].substr(2),
                                parseFloat(currentStocksChart.get(chart_index).close).toFixed(2)+"/"+parseFloat(currentStocksChart.get(chart_index).open).toFixed(2),
                                 " Vol: "+addCommas(currentStocksChart.get(chart_index).volume)].join('  '))

        chartDot.x = Math.floor(info[1]-chartDot.width/2)
        chartDot.y = Math.floor(info[2]-chartDot.height/2)
    }

}














